#ifndef _ASM_SW64_CHIP_QEMU_IO_H
#define _ASM_SW64_CHIP_QEMU_IO_H

#include <asm/platform.h>

#define IO_BASE			(0x1UL << 47)
#define PCI_BASE		(0x1UL << 43)
#define PCI_64BIT_MEMIO		(0x1UL << 39)
#define PCI_IO_PIU0		(0x2UL << 32)
#define PCI_IO_PIU1		(0x3UL << 32)
#define PCI_RC_CFG		(0x5UL << 32)
#define PCI_EP_CFG		(0x3UL << 33)
#define PCI_LEGACY_IO		(0x1UL << 32)
#define PCI_MEM_UNPRE		0x0UL
#define PCI_32BIT_MEMIO		(0xc0000000UL)
#define PCI_32BIT_MEMIO_SIZE	(0x20000000UL)
#define PCI_LEGACY_IO_SIZE	(0x100000000UL)
#define PCI_64BIT_MEMIO_SIZE	(0x8000000000UL)

#define IO_RC_SHIFT		40

/* MSIConfig */
#define MSICONFIG_VALID		(0x1UL << 63)
#define MSICONFIG_EN		(0x1UL << 62)
#define MSICONFIG_VECTOR_SHIFT	10


#define IO_MARK_BIT		47
#define CG_NUM			4
#define CG_ONLINE_IOADDR	((0x1UL<<47) | (0x5UL<<36) | (0x0UL<<7) | PAGE_OFFSET)

#define CORE_SLEEP_STAT_IOADDR	((0x1UL<<47) | (0x6UL<<36) | (0x1UL<<7) | PAGE_OFFSET)
#define CORE_START_IOADDR	((0x1UL<<47) | (0x5UL<<36) | (0xfUL<<7) | PAGE_OFFSET)

#define SW_PCI0_BUS		0x0
#define PCI0_BUS		SW_PCI0_BUS

#define MAX_NR_NODES		0x2
#define MAX_NR_RCS		0x6

#define SW_PCI_DEBUG		0
#if SW_PCI_DEBUG
#define PCIINFO(fmt, args...)	printk(fmt, ##args)
#else
#define PCIINFO(fmt, args...)
#endif

#define MCU_BASE		(0x3UL << 36)
#define INTPU_BASE		(0x2aUL << 32)
#define IO_NODE_SHIFT		44

#ifdef CONFIG_SUBARCH_C3A
#define INIT_CTL		((0x5UL << 36) | 0x680UL)
#define LONG_TIME		((0x5UL << 36) | 0x1180UL)
#endif

#ifdef CONFIG_SUBARCH_C3B
#define INIT_CTL		(MCU_BASE | 0x680UL)
#define LONG_TIME		(INTPU_BASE | 0x180UL)
#endif

/*-----------------------addr----------------------------------------------*/
/* MCU REG */
enum {
	CORE_ONLINE =		MCU_BASE | 0x780UL,
	CPU_ID =		MCU_BASE | 0x900UL,
	SI_FAULT_INT_EN =	MCU_BASE | 0x3200UL,
	MC_ONLINE =		MCU_BASE | 0x3780UL,
};

/* PIU IOR0 */
enum {
	EPDMABAR =		0x80UL,
	MSICONFIG0 =		0x2200UL,
};

/* RC */
enum {
	RC_PRIMARY_BUS =	0x300UL,
};
/*--------------------------------------------------------------------------*/

extern unsigned long sw_io_read(unsigned long node, unsigned long reg);
extern void sw_io_write(unsigned long node, unsigned long reg, unsigned long data);

#endif   /*_ASM_SW64_CHIP_QEMU_IO_H*/
