#ifndef _ASM_SW64_MMZONE_H
#define _ASM_SW64_MMZONE_H

#include <asm/smp.h>

struct bootmem_data_t; /* stupid forward decl. */

/*
 * Following are macros that are specific to this numa platform.
 */

extern pg_data_t node_data[];

extern int sw_pa_to_nid(unsigned long pa);
extern unsigned long sw_node_mem_start(int nid);
extern unsigned long sw_node_mem_size(int nid);

#define sw_64_pa_to_nid(pa)	sw_pa_to_nid(pa)
#define node_mem_start(nid)	sw_node_mem_start(nid)
#define node_mem_size(nid)	sw_node_mem_size(nid)

#define pa_to_nid(pa)		sw_64_pa_to_nid(pa)
#define NODE_DATA(nid)		(&node_data[(nid)])

#define node_localnr(pfn, nid)	((pfn) - NODE_DATA(nid)->node_start_pfn)

#define PLAT_NODE_DATA_LOCALNR(p, n) \
	(((p) >> PAGE_SHIFT) - PLAT_NODE_DATA(n)->gendata.node_start_pfn)

#ifdef CONFIG_DISCONTIGMEM

/*
 * Following are macros that each numa implementation must define.
 */

/*
 * Given a kernel address, find the home node of the underlying memory.
 */
#define kvaddr_to_nid(kaddr)	pa_to_nid(__pa(kaddr))

/*
 * Given a kaddr, LOCAL_BASE_ADDR finds the owning node of the memory
 * and returns the kaddr corresponding to first physical page in the
 * node's mem_map.
 */
#define LOCAL_BASE_ADDR(kaddr) \
	((unsigned long)__va(NODE_DATA(kvaddr_to_nid(kaddr))->node_start_pfn << PAGE_SHIFT))

/* XXX: FIXME -- wli */
#define kern_addr_valid(kaddr)	(0)

#define virt_to_page(kaddr)	pfn_to_page(__pa(kaddr) >> PAGE_SHIFT)

#define pmd_page(pmd)		(pfn_to_page(pmd_val(pmd) >> _PTE_FLAGS_BITS))
#define pud_page(pud)		(pfn_to_page(pud_val(pud) >> _PTE_FLAGS_BITS))
#define pgd_page(pgd)		(pfn_to_page(pgd_val(pgd) >> _PTE_FLAGS_BITS))
#define pte_pfn(pte)		(pte_val(pte) >> _PTE_FLAGS_BITS)
#define pmd_pfn(pmd)		(pmd_val(pmd) >> _PTE_FLAGS_BITS)

#define mk_pte(page, pgprot)							\
({										\
	pte_t pte;								\
	unsigned long pfn;							\
										\
	pfn = page_to_pfn(page) << _PTE_FLAGS_BITS;				\
	pte_val(pte) = pfn | pgprot_val(pgprot);				\
										\
	pte;									\
})

#define pte_page(x)								\
({										\
	unsigned long kvirt;							\
	struct page *__xx;							\
										\
	kvirt = (unsigned long)__va(pte_val(x) >> (_PTE_FLAGS_BITS-PAGE_SHIFT));\
	__xx = virt_to_page(kvirt);						\
										\
	__xx;									\
})

#define page_to_pa(page)	(page_to_pfn(page) << PAGE_SHIFT)

#define pfn_to_nid(pfn)		pa_to_nid(((u64)(pfn) << PAGE_SHIFT))
#define pfn_valid(pfn) \
	(((pfn) - node_start_pfn(pfn_to_nid(pfn))) < node_spanned_pages(pfn_to_nid(pfn)))

#define virt_addr_valid(kaddr)	pfn_valid((__pa(kaddr) >> PAGE_SHIFT))

#endif /* CONFIG_DISCONTIGMEM */

#endif /* _ASM_SW64_MMZONE_H */
