#ifndef _STARDBY_CDSW_H
#define _STARDBY_CDSW_H

#include <asm/swio.h>

#define READ_MAX_COUNT  100

static inline int
get_rc_num(struct pci_dev *pdev, unsigned int *pci_index)
{
        struct pci_controller *hose = pdev->bus->sysdata;
	*pci_index = hose->index;
	return 0;
}

static inline void
set_pme_config(struct pci_dev *pdev, unsigned int pci_index, u8 pin)
{
     unsigned int rc_root_control, temp_data, shift_bit;
     unsigned long pme_int_config;
     unsigned long devint_wken;
     unsigned long devint_mis_rpten;
     u16 pci_interrupt, interrupt_pin;
     sw_read_rc_conf(0, pci_index, 0x1180, &rc_root_control);
     rc_root_control |= 0x8;
     sw_write_rc_conf(0, pci_index, 0x1180, rc_root_control);

     devint_wken = DEVINT_WKEN;
     devint_mis_rpten = DEVINT_MIS_RPTEN;
     pci_read_config_word(pdev, 0x3c, &pci_interrupt);
     interrupt_pin = (pci_interrupt >> 8) & 0xFF;
     if(pci_index == 0) {
         shift_bit = 4;
     } else {
         shift_bit = 12;
     }
     shift_bit = shift_bit + (interrupt_pin -1);
     temp_data = *(unsigned int*) devint_wken;
     *(unsigned int*) devint_wken = temp_data | (1 << shift_bit);

     *(unsigned int*) devint_mis_rpten = 0x39f1f0;

     shift_bit = 4;
     shift_bit = shift_bit + (interrupt_pin -1);
     sw_read_piu_ior0(0, pci_index,  0xa500, &pme_int_config);
     pme_int_config |= 0xc000000000000000;
     pme_int_config |= (1 << shift_bit);
     sw_write_piu_ior0(0, pci_index, 0xa500, pme_int_config);
}


static inline int
cdsw_device_pci_suspend(struct pci_dev *pdev)
{
   u16 pmcsr;
   unsigned int pci_index, pme_status;
   int i;
   if (!pdev->pme_support)
       return 0;
   get_rc_num(pdev, &pci_index);
   set_pme_config(pdev, pci_index, pdev->pin);
   pci_read_config_word(pdev, pdev->pm_cap + PCI_PM_CTRL, &pmcsr);
   /* enable PME# */
   pmcsr |=  PCI_PM_CTRL_PME_ENABLE;
   pci_write_config_word(pdev, pdev->pm_cap + PCI_PM_CTRL, pmcsr);

   for(i = 0; i < READ_MAX_COUNT; ++i) {
       sw_write_rc_conf(0, pci_index, 0X1200, pme_status);
       sw_read_rc_conf(0, pci_index, 0x1200, &pme_status);
       if((pme_status & 0x10000) == 0) {
           return 0;
       }
   }
   printk("%s: PMSstatus bit of rc root status cann't be cleared\n", __func__);

   pci_read_config_word(pdev, pdev->pm_cap + PCI_PM_CTRL, &pmcsr);
   /* disable PME# */
   pmcsr &=  ~PCI_PM_CTRL_PME_ENABLE;
   pci_write_config_word(pdev, pdev->pm_cap + PCI_PM_CTRL, pmcsr);

   return -EIO;
}


static inline int
cdsw_device_pci_resume(struct pci_dev *pdev)
{
   u16 pmcsr;
   unsigned int pme_status = 0;
   int pci_index, i;
   if (!pdev->pme_support)
       return 0;
   get_rc_num(pdev, &pci_index);

   pci_read_config_word(pdev, pdev->pm_cap + PCI_PM_CTRL, &pmcsr);
   /* disable PME# */
   pmcsr &=  ~PCI_PM_CTRL_PME_ENABLE;
   pci_write_config_word(pdev, pdev->pm_cap + PCI_PM_CTRL, pmcsr);

   sw_read_rc_conf(0, pci_index, 0x1200, &pme_status);
   for(i = 0; i < READ_MAX_COUNT; ++i) {
       sw_write_rc_conf(0, pci_index, 0X1200, pme_status);
       sw_read_rc_conf(0, pci_index, 0x1200, &pme_status);
       if((pme_status & 0x10000) == 0) {
           return 0;
       }
   }
   printk("%s: PMSstatus bit of rc root status cann't be cleared\n", __func__);
   return -EIO;
}
#endif
