/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _ASM_SW64_TOPOLOGY_H
#define _ASM_SW64_TOPOLOGY_H

#include <linux/smp.h>
#include <linux/threads.h>
#include <linux/cpumask.h>

#define THREAD_ID_SHIFT	5
#define THREAD_ID_MASK	1

struct cpu_topology {
	int node_id;
	int core_id;
	int thread_id;
	int llc_id;
	cpumask_t thread_sibling;
	cpumask_t core_sibling;
	cpumask_t llc_sibling;
};

extern struct cpu_topology cpu_topology[NR_CPUS];

#define topology_physical_package_id(cpu)	(cpu_topology[cpu].node_id)
#define topology_core_id(cpu)			(cpu_topology[cpu].core_id)
#define topology_core_cpumask(cpu)		(&cpu_topology[cpu].core_sibling)
#define topology_sibling_cpumask(cpu)		(&cpu_topology[cpu].thread_sibling)
#define topology_llc_cpumask(cpu)		(&cpu_topology[cpu].llc_sibling)

void init_cpu_topology(void);
void store_cpu_topology(int cpuid);
void remove_cpu_topology(int cpuid);
const struct cpumask *cpu_coregroup_mask(int cpu);

extern int sw_cpuid_to_nid(int cpuid);
static inline int cpu_to_node(int cpu)
{
	int node;

	node = sw_cpuid_to_nid(cpu);

#ifdef DEBUG_NUMA
	BUG_ON(node < 0);
#endif

	return node;
}
#ifdef CONFIG_NUMA
extern const struct cpumask *cpumask_of_node(int node);
#define parent_node(node) (node)
#define cpumask_of_pcibus(bus)	(cpu_online_mask)
#endif /* !CONFIG_NUMA */
# include <asm-generic/topology.h>

#endif /* _ASM_SW64_TOPOLOGY_H */
